#!/usr/bin/perl
use strict;
use Capture::Tiny 'capture';
use Term::ProgressBar 2.00;

# Compares all icon files in first directory with partners in second directory, whether their pixel representation is equal.
# Use this to compare existing breeze-icons/icons-dark/ and icons-dark_generated/, where the latter is autogenerated from breeze-icons/icons/.
# Usage:
# $ ./compare-light-dark.pl icons-dark/ icons-dark_generated/

my $first_dir = shift @ARGV;
my $second_dir = shift @ARGV;

my @files_without_partner;
my @files_not_equal;

# Progress bar: Comparing all icons takes very long, so show a progress bar.
my $files_to_change = `./count-icons.pl $first_dir` * 1;
my $progress_bar = Term::ProgressBar->new({count=>$files_to_change, ETA=>'linear'});
my $progress_bar_pos = 0;

# Compares two images using ImageMagick compare.
# Args: (filepath, filepath)
# Returns: float; 0 if equal, >0 otherwise
sub compare
{
    my ($first, $second) = @_;

    # If symlink pointing to same relative target, return 0
    if (-l $first and -l $second and (readlink $first eq readlink $second)) {
        return 0;
    }

    my (undef, $diff) = capture { system("compare -metric mae $first $second /dev/null"); };
    return $diff * 1;
}

# Traverses one directory:
# For icon files calls process_icon, for directories calls process_dir
# Arg 1: path
sub process_dir
{
    my $dir = shift;
    for my $file (glob $dir . "/*") {
        if (-l $file) {
            next;
        } elsif (-d -r -x $file) {
            &process_dir($file);
        } elsif (-f -r $file) {
            &process_icon($file);
            $progress_bar->update(++$progress_bar_pos);
        } else {
            warn "Can not work with path $file";
        }
    }
}

# Compares one icon file by it’s partner in the second directory.
# If icon and parter are differend, appends path to @files_not_equal.
# If partner does not exist, appends path to @files_without_partner.
# Arg 1: path
sub process_icon
{
    my $file = shift;
    my $partner = $file;
    ($partner =~ s/\A$first_dir/$second_dir/) or warn "Directory substitution failed!";

    if (!(-e -f -r $partner)) {
        push @files_without_partner, $file;
        return;
    }

    my $difference = &compare($file, $partner);
    if ($difference != 0) {
        push @files_not_equal, [$file, $difference];
    }
}

if (-d -r -x $first_dir and !(-l $first_dir)) {
    &process_dir($first_dir);
} else {
    die "Bad first_dir given: $first_dir";
}

if (@files_without_partner) {
    printf "  == Files in $first_dir without partner in $second_dir: (Count: %d) ==\n", scalar(@files_without_partner);
    for (@files_without_partner) {
        print "$_\n";
    }
} else {
    print "  == All icons in $first_dir have partner in $second_dir! ==\n";
}

if (@files_not_equal) {
    printf "  == Different files: (Count: %d) ==\n", scalar(@files_not_equal);
    for (@files_not_equal) {
        printf "%8f %s\n", $$_[1], $$_[0];
    }
} else {
    print "  == All icons in $first_dir and $second_dir equal! ==\n";
}
